package poo.util;

import java.util.ArrayList;

/**
 * Esta e' uma classe abstrata da qual os dispositivos Computador
 * Switch e Hub serao herdados
 *
 * @author Emerson Ribeiro de Mello
 */
public abstract class Dispositivo {
   // identificador unico do dispositivo
   private int ID;
   // Fila de pacotes que chegam ao dispositivo   
   private ArrayList<Pacote> fila;
   // contabiliza o total de pacotes que foram descartados
   private int totalDeDescartes;
   // contabiliza o total de pacotes que foram processados
   private int totalDePacotesProcessados;

   /**
    * Construtor da classe abstrata Dispositivo
    * @param ID - identificador do dispositivo
    */
   public Dispositivo(int ID) {
      this.ID = ID;
      this.fila = new ArrayList<Pacote>();
      this.totalDeDescartes = 0;
      this.totalDePacotesProcessados = 0;
   }

   /*************************************************************************/
   /*      Metodos abstract que devem ser implementados nas subclasses      */
   /*************************************************************************/

   /**
    * Adiciona pacote a fila do dispositivo. Esse metodo sera' utilizado durante
    * a simulacao para adicionar pacotes nas filas dos dispositivos. Cabe a cada
    * dispositivo implementar um sistema contra loops.
    * Dica: Chame dentro deste metodo o metodo 'carregarFila(Pacote) para evitar
    * ter que reescrever codigo
    * @param p pacote a ser adicionado
    * @return retorna true se o pacote for adicionado e false caso contrario
    */
   public abstract boolean adicionarPacote(Pacote p);

   /**
    * Liga um dipositivo a outro
    * @param disp o dispositivo que sera' conectado
    * @return retorna true se ele conectar, falso caso nao tenha porta 
    * disponivel
    */
   public abstract boolean conectar(Dispositivo disp);

   /**
    * Para desconectar um dispositivo
    * @param disp o dispositivo que sera' desconectado
    * @return retorna true se ocorrer com sucesso, false caso contrario
    */
   public abstract boolean desconectar(Dispositivo disp);

   /**
    * Metodo abstrato que devera' ser implementado nas subclasses.
    * Ele tem o objetivo de processar os pacotes que estao na fila.
    */
   public abstract void processar();

   /*************************************************************************/
   
   
   /**
    * Remove um pacote da fila do dispositivo.
    * @return retorna o pacote caso a fila nao esteja vazia e null caso contrario
    */
   protected Pacote removerPacote() {
      if (!fila.isEmpty()) {
         return this.fila.remove(0);
      } else {
         return null;
      }
   }

   /**
    * Se o pacote processado nao tiver como destino o Dispositivo em questao,
    * este sera' descartado e sera' contabilizado como pacote descartado
    */
   protected void adicionaPacoteDescartado() {
      this.totalDeDescartes++;
   }

   /**
    * Se o pacote processado tiver como destino o Dispositivo em questao,
    * este sera' sera' contabilizado como pacote processado
    */
   protected void adicionaPacoteProcessado() {
      this.totalDePacotesProcessados++;
   }

   /**
    * Retorna o identificador do dispositivo
    * @return - identificador do dispositivo
    */
   public int obtemID() {
      return this.ID;
   }

   /**
    * Indica se a fila de pacotes esta' vazia ou nao
    * @return - retorna true ou false
    */
   public boolean filaVazia() {
      return this.fila.isEmpty();
   }

   /**
    * Adiciona pacotes na fila do dispositivo.
    * @param p pacote a ser adicionado
    * @return retorna true se o pacote for adicionado e false caso contrario
    */
   public boolean carregarFila(Pacote p) {
      if (fila.size() < 10) {
         this.fila.add(p);
         return true;
      } else {
         this.totalDeDescartes++;
         return false;
      }
   }

   /**
    * Retorna o total de pacotes que foram descartados pelo dispositivo
    * @return inteiro contendo o total de descartes
    */
   public int obtemTotalDeDescartes() {
      return totalDeDescartes;
   }

   /**
    * Retorna o total de pacotes que foram processados pelo dispositivo
    * @return inteiro contendo o total de pacotes processados
    */
   public int obtemTotalDePacotesProcessados() {
      return totalDePacotesProcessados;
   }

   /**
    * Retorna um String indicando a quantidade de pacotes que foram processados
    * e descartados pelo Dispositivo
    * @return String
    */
   @Override
   public String toString() {
      return "[" + this.ID + "] processou: " + this.totalDePacotesProcessados +
              "\t descartou: " + this.totalDeDescartes;
   }
}//fim da classe

