package poo.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;

/**
 * Esta classe tem por objetivo trabalhar com arquivos. Esta possui duas principais
 * funcoes:
 * 1 - Carregar de um arquivo texto (devidamente formatado) o trafego que sera'
 *    usado na simulacao
 * 2 - Gerar um arquivo texto tendo cada linha o seguinte formato:
 *       origem>destino
 *  Origem e Destino sao numeros inteiros. Este arquivo servira' de base para
 * as simulacoes.
 * 
 * @author Emerson Ribeiro de Mello
 */
public abstract class ManipuladorDeArquivos {

   //semente para gerar numeros pseudo-aleatorios
   private static int semente = 0; 

   /**
    * Abre um arquivo texto e retorna um objeto BufferedReader
    * @param nomeDoArquivo - caminho com o nome do arquivo
    * @return retorna objeto BufferedReader
    */
   private static BufferedReader abrirArquivo(String nomeDoArquivo) {
      try {
         File arquivo = new File(nomeDoArquivo);

         if (arquivo.exists()) {//verifica se o arquivo existe

            FileReader frArquivo = new FileReader(arquivo);
            BufferedReader buffer = new BufferedReader(frArquivo);

            return buffer;

         } else {//caso o arquivo nao exista
            System.err.println("Arquivo inexistente...");
         }
      } catch (Exception exception) {//caso ocorra algum erro
         System.err.println("Erro: " + exception.toString());
      }
      // retorna null caso o arquivo nao exista ou caso ocorra algum erro
      // ao tentar abrir o arquivo
      return null;
   }

   /**
    * Carrega o trafego de um arquivo texto para a lista de dispositivos
    * @param nomeDoArquivo nome do arquivo que contem o trafego
    * @param lista lista de dispositivos que irao receber o trafego
    */
   public static void carregarTrafego(String nomeDoArquivo, Dispositivo lista[]) {
      String linha = null;

      BufferedReader arquivo = abrirArquivo(nomeDoArquivo);

      try {
         if (arquivo != null) {
            do {
               //le o arquivo texto linha a linha
               linha = arquivo.readLine();

               if (linha != null) {//so' continua se a linha nao for nula
                  //divide a linha em duas partes: origem > destino
                  String[] partes = linha.split(">");

                  //iniciando variaveis
                  int origem = Integer.MAX_VALUE;
                  int destino = Integer.MAX_VALUE;

                  try {
                     //convertendo String para int
                     origem = new Integer(partes[0]).intValue();
                     destino = new Integer(partes[1]).intValue();
                     
                  } catch (Exception exception) {
                     System.err.println("A linha atual nao esta' no formato correto." +
                             "\n origem e destino devem ser numeros inteiros." +
                             "\n Ex: origem>destino " + "\n" + exception.toString());
                  }

                  if ((origem < lista.length) && (destino < lista.length)) {
                     Pacote p = new Pacote(origem, destino);

                     if (!lista[origem].carregarFila(p)) {
                        System.err.println("Disp[" + origem + "]: fila cheia!");
                     }
                  } else {
                     System.err.println("Origem e/ou destino fora da faixa permitida.");
                  }
               }
            } while (linha != null);
            //fechando o arquivo
            arquivo.close();
         }
      } catch (Exception ex) {
         System.err.println("Erro: " + ex.toString());
      }
   }

   /**
    * Este metodo tem o objetivo de gerar um arquivo com trafego, de forma
    * aleatoria
    * @param nomeDoArquivo nome do arquivo que sera' criado
    * @param totalDeComputadores total de computadores presente na rede
    * @param totalDePacotes total de pacotes que serao gerados
    */
   public static void gerarTrafego(String nomeDoArquivo, int totalDeComputadores, int totalDePacotes) {
      //para gerar trafego aleatorio
      Random aleatorio = new Random(semente);

      int origem, destino;
      HashSet<String> trafego = new HashSet<String>();

      BufferedWriter buffer = gravarArquivo(nomeDoArquivo);

      if (buffer != null) {
         try {

            while (totalDePacotes > 0) {

               origem = aleatorio.nextInt(totalDeComputadores);

               do {
                  //para evitar que o destino seja igual a origem
                  destino = aleatorio.nextInt(totalDeComputadores);
               } while (origem == destino);

               String linha = origem + ">" + destino;

               // so' adiciona a linha se ela ainda nao existir no conjunto
               // isso evita adicionar linhas duplicadas
               if (trafego.add(linha)) {
                  totalDePacotes--;
               }
            }

            //gravando o trafego no arquivo
            Iterator it = trafego.iterator();
            while (it.hasNext()) {
               buffer.write((String) it.next() + "\n");
            }
            //fechando o arquivo
            buffer.close();
         } catch (IOException ex) {
            System.err.println("Erro: " + ex.toString());
         }
      } else {
         System.err.println("Nao foi possivel criar o arquivo: " + nomeDoArquivo);
      }
   }

   /**
    * Cria um arquivo para armazenar o trafego
    * @param nomeDoArquivo nome do arquivo que sera' criado
    * @return retorna um objeto BufferedWriter
    */
   private static BufferedWriter gravarArquivo(String nomeDoArquivo) {
      BufferedWriter buffer = null;
      try {
         File arquivo = new File(nomeDoArquivo);
         FileWriter fwArquivo = null;

         // Verifica se o arquivo existe
         // se nao existir, ele cria o arquivo
         // se existir, ele aborta o processo
         if (!arquivo.exists()) {
            fwArquivo = new FileWriter(arquivo);
         } else {
            throw new IOException("O arquivo ja' existe");
         }

         buffer = new BufferedWriter(fwArquivo);

      } catch (Exception exception) {
         System.err.println("Erro: " + exception.toString());
      }
      return buffer;
   }
}//fim da classe

